Imports System.ComponentModel
Imports Facebook
Imports FbAuthWpf.Common

Namespace FbAuthWpf.Presenters
    Public Class MainWindowPresenter
        Implements INotifyPropertyChanged

#Region "Private members"
        Private ReadOnly _view As MainWindow
        Private _token As String
        Private _status As String
        Private _userName As String
        Private _userPicture As String
        Private _photoPath As String
        Private _showBrowser As Boolean
        Private _isLogged As Boolean
#End Region

        Public Sub New(ByVal view As MainWindow)
            _view = view
            _showBrowser = False
            ResetUserInfo()
        End Sub

        ''' <summary>
        ''' Gets and sets the FB token of the current user.
        ''' </summary>
        Public Property Token As String
            Get
                Return _token
            End Get
            Set(ByVal value As String)
                _token = value
                OnPropertyChanged("Token")
            End Set
        End Property

        ''' <summary>
        ''' Gets and sets the name of the currently logged user.
        ''' </summary>
        Public Property UserName As String
            Get
                Return _userName
            End Get
            Set(ByVal value As String)
                _userName = value
                OnPropertyChanged("UserName")
            End Set
        End Property

        ''' <summary>
        ''' Gets and sets the URL of the FB user picture.
        ''' </summary>
        Public Property UserPicture As String
            Get
                Return _userPicture
            End Get
            Set(ByVal value As String)
                _userPicture = value
                OnPropertyChanged("UserPicture")
            End Set
        End Property

        ''' <summary>
        ''' Gets and sets the text to post.
        ''' </summary>
        Public Property Status As String
            Get
                Return _status
            End Get
            Set(ByVal value As String)
                _status = value
                OnPropertyChanged("Status")
            End Set
        End Property

        ''' <summary>
        ''' Gets and sets the photo path to post.
        ''' </summary>
        Public Property PhotoPath As String
            Get
                Return _photoPath
            End Get
            Set(ByVal value As String)
                _photoPath = value
                OnPropertyChanged("PhotoPath")
            End Set
        End Property

        ''' <summary>
        ''' Shows and hides programmatically the web browser used to log to FB.
        ''' </summary>
        Public Property ShowBrowser As Boolean
            Get
                Return _showBrowser
            End Get
            Set(ByVal value As Boolean)
                _showBrowser = value
                OnPropertyChanged("ShowBrowser")
            End Set
        End Property

        ''' <summary>
        ''' Shows and hides programmatically the web browser used to log to FB.
        ''' </summary>
        Public Property IsLogged As Boolean
            Get
                Return _isLogged
            End Get
            Set(ByVal value As Boolean)
                _isLogged = value
                OnPropertyChanged("IsLogged")
            End Set
        End Property

        ''' <summary>
        ''' Triggers the authentication process towards Facebook
        ''' </summary>
        Public Sub Login()
            Dim loginUrl = FbHelpers.GetLoginUrl
            ShowBrowser = True
            _view.Browser.Navigate(loginUrl)
        End Sub

        ''' <summary>
        ''' 
        ''' </summary>
        Public Sub Logout()
            ResetUserInfo()
        End Sub

        ''' <summary>
        ''' 
        ''' </summary>
        Public Sub PostUpdate()
            If String.IsNullOrEmpty(PhotoPath) Then
                FbHelpers.Post(Token, Status)
            Else
                FbHelpers.PostWithPhoto(Token, Status, PhotoPath)
            End If

            Status = String.Empty
            PhotoPath = String.Empty
        End Sub

        ''' <summary>
        ''' 
        ''' </summary>
        Public Sub BrowseForJpegFile()
            Dim dialog = New Microsoft.Win32.OpenFileDialog With {.DefaultExt = ".jpg", .Filter = "JPEG |*.jpg"}
            Dim result = dialog.ShowDialog
            If result = True Then
                PhotoPath = dialog.FileName
            End If
        End Sub

        ''' <summary>
        ''' 
        ''' </summary>
        ''' <param name="oauthResult"></param>
        Public Sub LoginSucceeded(ByVal oauthResult As FacebookOAuthResult)
            ShowBrowser = False
            Dim token = FbHelpers.GetAccessToken(oauthResult)
            Me.Token = token
            Dim user As Object = FbHelpers.GetUser(token)

            UserName = String.Format("{0} {1}", user.first_name, user.last_name)
            UserPicture = user.picture.ToString
            IsLogged = True
        End Sub

        ''' <summary>
        ''' 
        ''' </summary>
        ''' <param name="result"></param>
        Public Sub LoginFailed(ByVal result As FacebookOAuthResult)
            ShowBrowser = False
            IsLogged = False
            MessageBox.Show(result.ErrorDescription)
        End Sub

#Region "Private"
        Private Sub ResetUserInfo()
            UserName = "No user currently logged in."
            UserPicture = String.Empty
            IsLogged = False
            Status = "Another post in the wall"
        End Sub
#End Region

#Region "INotifyPropertyChanged"
        Public Event PropertyChanged As PropertyChangedEventHandler Implements INotifyPropertyChanged.PropertyChanged
        Public Sub OnPropertyChanged(ByVal propertyString As String)
            Dim handler As PropertyChangedEventHandler = PropertyChangedEvent
            If Not handler Is Nothing Then
                handler(Me, New PropertyChangedEventArgs(propertyString))
            End If
        End Sub
#End Region

    End Class
End Namespace
